#ifndef BITMAP_H_INCLUDED
#define BITMAP_H_INCLUDED

#include "stuff.h"

class Bitmap {
    bool isSubBitmap;
    BITMAP* bmp;
    int* refcnt;

    void _free();
    void _copy(const Bitmap& other);
public:
    Bitmap() : isSubBitmap(false), bmp(NULL), refcnt(NULL) { }
    Bitmap(BITMAP* bitmap, bool own) : isSubBitmap(false), bmp(bitmap), refcnt(new int((own?1:2))) { }
    Bitmap(int w, int h) : isSubBitmap(false), bmp(NULL), refcnt(NULL) { make(w,h); }
    Bitmap(string filename) : isSubBitmap(false), bmp(NULL), refcnt(NULL) { load(filename); }
    Bitmap(const Bitmap& other) : isSubBitmap(false), bmp(NULL), refcnt(NULL) { _copy(other); }
    ~Bitmap() { _free(); }

    Bitmap& operator =(const Bitmap& other) { _copy(other); return *this; }

    bool make(int w, int h);

    // drawing primitives
    void clear(int color = 0) { clear_to_color(bmp, color); }
    void putpixel(int x, int y, int color) { ::putpixel(bmp, x, y, color); }
    int getpixel(int x, int y) const { return ::getpixel(bmp, x, y); }
    void line(int x1, int y1, int x2, int y2, int color) { ::line(bmp, x1, y1, x2, y2, color); }
    void polygon(int npoints, const int* points, int color) { ::polygon(bmp, npoints, points, color); }
    void rect(int x1, int y1, int x2, int y2, int color) { ::rect(bmp, x1, y1, x2, y2, color); }
    void fillrect(int x1, int y1, int x2, int y2, int color) { ::rectfill(bmp, x1, y1, x2, y2, color); }
    void circle(int x, int y, int r, int color) { ::circle(bmp, x, y, r, color); }
    void fillcircle(int x, int y, int r, int color) { ::circlefill(bmp, x, y, r, color); }
    void ellipse(int x, int y, int rx, int ry, int color) { ::ellipse(bmp, x, y, rx, ry, color); }
    void fillellipse(int x, int y, int rx, int ry, int color) { ::ellipsefill(bmp, x, y, rx, ry, color); }
    void floodfill(int x, int y, int color) { ::floodfill(bmp, x, y, color); }

    // draw sprites
    void draw(const Bitmap& other, int x, int y) { ::blit(other.bmp, bmp, 0, 0, x, y, other.bmp->w, other.bmp->h); }
    void draw_sprite(const Bitmap& other, int x, int y) { draw_sprite_ex(bmp, other.bmp, x, y, DRAW_SPRITE_NORMAL, DRAW_SPRITE_NO_FLIP); }
    void draw_sprite_flip(const Bitmap& other, int x, int y) { draw_sprite_ex(bmp, other.bmp, x, y, DRAW_SPRITE_NORMAL, DRAW_SPRITE_H_FLIP); }
    void draw_to_screen() { acquire_screen(); ::blit(bmp, screen, 0, 0, 0, 0, bmp->w, bmp->h); release_screen(); }

    // textout
    void textout(FONT* f, string str, int x, int y, int color = -1, int bg = -1)
        { textout_ex(bmp, f, str.c_str(), x, y, color, bg); }
    void textout_center(FONT* f, string str, int x, int y, int color = -1, int bg = -1)
        { textout_centre_ex(bmp, f, str.c_str(), x, y, color, bg); }
    void textout_right(FONT* f, string str, int x, int y, int color = -1, int bg = -1)
        { textout_right_ex(bmp, f, str.c_str(), x, y, color, bg); }
    void debug_print(string str) { textout_ex(bmp, font, str.c_str(), 0, bmp->h-text_height(font), -1, 0); }

    // other
    bool load(string filename);
    Bitmap sub(int x, int y, int w, int h);

    BITMAP* ptr() { return bmp; }
    BITMAP* operator->() { return ptr(); }
};

inline int color(int r, int g, int b) { return makecol32(r, g, b); }
inline int color(int r, int g, int b, int a) { return makeacol32(r, g, b, a); }
inline bool in_rect(int x1, int y1, int x2, int y2, int x, int y) {
	return (x >= x1 && x <= x2 && y >= y1 && y <= y2); }


#endif // BITMAP_H_INCLUDED
